@@ -22,6 +22,7 @@ import logging
 import os
 import sys
 
+from novaclient.v2 import servers
 from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
@@ -29,11 +30,6 @@ from osc_lib import utils
 from oslo_utils import timeutils
 import six
 
-try:
-    from novaclient.v2 import servers
-except ImportError:
-    from novaclient.v1_1 import servers
-
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 
@@ -114,6 +110,16 @@ def _get_ip_address(addresses, address_type, ip_address_family):
         )
 
 
+def _prefix_checked_value(prefix):
+    def func(value):
+        if ',' in value or '=' in value:
+            msg = _("Invalid argument %s, "
+                    "characters ',' and '=' are not allowed") % value
+            raise argparse.ArgumentTypeError(msg)
+        return prefix + value
+    return func
+
+
 def _prep_server_detail(compute_client, image_client, server):
     """Prepare the detailed server dict for printing
 
@@ -197,13 +203,19 @@ class AddFixedIP(command.Command):
         parser.add_argument(
             "server",
             metavar="<server>",
-            help=_("Server (name or ID) to receive the fixed IP address"),
+            help=_("Server to receive the fixed IP address (name or ID)"),
         )
         parser.add_argument(
             "network",
             metavar="<network>",
-            help=_("Network (name or ID) to allocate "
-                   "the fixed IP address from"),
+            help=_(
+                "Network to allocate the fixed IP address from (name or ID)"
+            ),
+        )
+        parser.add_argument(
+            "--fixed-ip-address",
+            metavar="<ip-address>",
+            help=_("Requested fixed IP address"),
         )
         return parser
 
@@ -213,10 +225,13 @@ class AddFixedIP(command.Command):
         server = utils.find_resource(
             compute_client.servers, parsed_args.server)
 
-        network = utils.find_resource(
-            compute_client.networks, parsed_args.network)
+        network = compute_client.api.network_find(parsed_args.network)
 
-        server.add_fixed_ip(network.id)
+        server.interface_attach(
+            port_id=None,
+            net_id=network['id'],
+            fixed_ip=parsed_args.fixed_ip_address,
+        )
 
 
 class AddFloatingIP(command.Command):
@@ -227,13 +242,18 @@ class AddFloatingIP(command.Command):
         parser.add_argument(
             "server",
             metavar="<server>",
-            help=_("Server (name or ID) to receive the floating IP address"),
+            help=_("Server to receive the floating IP address (name or ID)"),
         )
         parser.add_argument(
             "ip_address",
             metavar="<ip-address>",
-            help=_("Floating IP address (IP address only) to assign "
-                   "to server"),
+            help=_("Floating IP address to assign to server (IP only)"),
+        )
+        parser.add_argument(
+            "--fixed-ip-address",
+            metavar="<ip-address>",
+            help=_("Fixed IP address to associate with this floating IP "
+                   "address"),
         )
         return parser
 
@@ -243,7 +263,41 @@ class AddFloatingIP(command.Command):
         server = utils.find_resource(
             compute_client.servers, parsed_args.server)
 
-        server.add_floating_ip(parsed_args.ip_address)
+        server.add_floating_ip(parsed_args.ip_address,
+                               parsed_args.fixed_ip_address)
+
+
+class AddPort(command.Command):
+    _description = _("Add port to server")
+
+    def get_parser(self, prog_name):
+        parser = super(AddPort, self).get_parser(prog_name)
+        parser.add_argument(
+            "server",
+            metavar="<server>",
+            help=_("Server to add the port to (name or ID)"),
+        )
+        parser.add_argument(
+            "port",
+            metavar="<port>",
+            help=_("Port to add to the server (name or ID)"),
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        compute_client = self.app.client_manager.compute
+
+        server = utils.find_resource(
+            compute_client.servers, parsed_args.server)
+
+        if self.app.client_manager.is_network_endpoint_enabled():
+            network_client = self.app.client_manager.network
+            port_id = network_client.find_port(
+                parsed_args.port, ignore_missing=False).id
+        else:
+            port_id = parsed_args.port
+
+        server.interface_attach(port_id=port_id, net_id=None, fixed_ip=None)
 
 
 class AddServerSecurityGroup(command.Command):
@@ -270,12 +324,11 @@ class AddServerSecurityGroup(command.Command):
             compute_client.servers,
             parsed_args.server,
         )
-        security_group = utils.find_resource(
-            compute_client.security_groups,
+        security_group = compute_client.api.security_group_find(
             parsed_args.group,
         )
 
-        server.add_security_group(security_group.id)
+        server.add_security_group(security_group['id'])
 
 
 class AddServerVolume(command.Command):
@@ -341,10 +394,13 @@ class CreateServer(command.ShowOne):
         disk_group.add_argument(
             '--volume',
             metavar='<volume>',
-            help=_(
-                'Create server using this volume as the boot disk '
-                '(name or ID)'
-            ),
+            help=_('Create server using this volume as the boot disk (name '
+                   'or ID).\n'
+                   'This option automatically creates a block device mapping '
+                   'with a boot index of 0. On many hypervisors (libvirt/kvm '
+                   'for example) this will be device vda. Do not create a '
+                   'duplicate mapping using --block-device-mapping for this '
+                   'volume.'),
         )
         parser.add_argument(
             '--flavor',
@@ -354,7 +410,7 @@ class CreateServer(command.ShowOne):
         )
         parser.add_argument(
             '--security-group',
-            metavar='<security-group-name>',
+            metavar='<security-group>',
             action='append',
             default=[],
             help=_('Security group to assign to this server (name or ID) '
@@ -393,18 +449,29 @@ class CreateServer(command.ShowOne):
         parser.add_argument(
             '--block-device-mapping',
             metavar='<dev-name=mapping>',
-            action='append',
-            default=[],
-            help=_('Map block devices; map is '
-                   '<id>:<type>:<size(GB)>:<delete_on_terminate> '
+            action=parseractions.KeyValueAction,
+            default={},
+            # NOTE(RuiChen): Add '\n' at the end of line to put each item in
+            #                the separated line, avoid the help message looks
+            #                messy, see _SmartHelpFormatter in cliff.
+            help=_('Create a block device on the server.\n'
+                   'Block device mapping in the format\n'
+                   '<dev-name>=<id>:<type>:<size(GB)>:<delete-on-terminate>\n'
+                   '<dev-name>: block device name, like: vdb, xvdc '
+                   '(required)\n'
+                   '<id>: UUID of the volume or snapshot (required)\n'
+                   '<type>: volume or snapshot; default: volume (optional)\n'
+                   '<size(GB)>: volume size if create from snapshot '
+                   '(optional)\n'
+                   '<delete-on-terminate>: true or false; default: false '
+                   '(optional)\n'
                    '(optional extension)'),
         )
         parser.add_argument(
             '--nic',
             metavar="<net-id=net-uuid,v4-fixed-ip=ip-addr,v6-fixed-ip=ip-addr,"
-                    "port-id=port-uuid>",
+                    "port-id=port-uuid,auto,none>",
             action='append',
-            default=[],
             help=_("Create a NIC on the server. "
                    "Specify option multiple times to create multiple NICs. "
                    "Either net-id or port-id must be provided, but not both. "
@@ -414,7 +481,35 @@ class CreateServer(command.ShowOne):
                    "v6-fixed-ip: IPv6 fixed address for NIC (optional), "
                    "none: (v2.37+) no network is attached, "
                    "auto: (v2.37+) the compute service will automatically "
-                   "allocate a network."),
+                   "allocate a network. Specifying a --nic of auto or none "
+                   "cannot be used with any other --nic value."),
+        )
+        parser.add_argument(
+            '--network',
+            metavar="<network>",
+            action='append',
+            dest='nic',
+            type=_prefix_checked_value('net-id='),
+            help=_("Create a NIC on the server and connect it to network. "
+                   "Specify option multiple times to create multiple NICs. "
+                   "This is a wrapper for the '--nic net-id=<network>' "
+                   "parameter that provides simple syntax for the standard "
+                   "use case of connecting a new server to a given network. "
+                   "For more advanced use cases, refer to the '--nic' "
+                   "parameter."),
+        )
+        parser.add_argument(
+            '--port',
+            metavar="<port>",
+            action='append',
+            dest='nic',
+            type=_prefix_checked_value('port-id='),
+            help=_("Create a NIC on the server and connect it to port. "
+                   "Specify option multiple times to create multiple NICs. "
+                   "This is a wrapper for the '--nic port-id=<pord>' "
+                   "parameter that provides simple syntax for the standard "
+                   "use case of connecting a new server to a given port. For "
+                   "more advanced use cases, refer to the '--nic' parameter."),
         )
         parser.add_argument(
             '--hint',
@@ -518,45 +613,64 @@ class CreateServer(command.ShowOne):
                                         'source_type': 'volume',
                                         'destination_type': 'volume'
                                         }]
-        for dev_map in parsed_args.block_device_mapping:
-            dev_name, dev_map = dev_map.split('=', 1)
-            if dev_map:
-                dev_map = dev_map.split(':')
-                if len(dev_map) > 0:
-                    mapping = {
-                        'device_name': dev_name,
-                        'uuid': utils.find_resource(
-                            volume_client.volumes,
-                            dev_map[0],
-                        ).id}
-                    # Block device mapping v1 compatibility
-                    if len(dev_map) > 1 and \
-                            dev_map[1] in ('volume', 'snapshot'):
-                        mapping['source_type'] = dev_map[1]
-                    else:
-                        mapping['source_type'] = 'volume'
-                    mapping['destination_type'] = 'volume'
-                    if len(dev_map) > 2:
-                        mapping['volume_size'] = dev_map[2]
-                    if len(dev_map) > 3:
-                        mapping['delete_on_termination'] = dev_map[3]
+        # Handle block device by device name order, like: vdb -> vdc -> vdd
+        for dev_name in sorted(six.iterkeys(parsed_args.block_device_mapping)):
+            dev_map = parsed_args.block_device_mapping[dev_name]
+            dev_map = dev_map.split(':')
+            if dev_map[0]:
+                mapping = {'device_name': dev_name}
+                # 1. decide source and destination type
+                if (len(dev_map) > 1 and
+                        dev_map[1] in ('volume', 'snapshot')):
+                    mapping['source_type'] = dev_map[1]
                 else:
-                    msg = _("Volume name or ID must be specified if "
-                            "--block-device-mapping is specified")
-                    raise exceptions.CommandError(msg)
-                block_device_mapping_v2.append(mapping)
+                    mapping['source_type'] = 'volume'
+                mapping['destination_type'] = 'volume'
+                # 2. check target exist, update target uuid according by
+                #    source type
+                if mapping['source_type'] == 'volume':
+                    volume_id = utils.find_resource(
+                        volume_client.volumes, dev_map[0]).id
+                    mapping['uuid'] = volume_id
+                elif mapping['source_type'] == 'snapshot':
+                    snapshot_id = utils.find_resource(
+                        volume_client.volume_snapshots, dev_map[0]).id
+                    mapping['uuid'] = snapshot_id
+                # 3. append size and delete_on_termination if exist
+                if len(dev_map) > 2 and dev_map[2]:
+                    mapping['volume_size'] = dev_map[2]
+                if len(dev_map) > 3 and dev_map[3]:
+                    mapping['delete_on_termination'] = dev_map[3]
+            else:
+                msg = _("Volume or snapshot (name or ID) must be specified if "
+                        "--block-device-mapping is specified")
+                raise exceptions.CommandError(msg)
+            block_device_mapping_v2.append(mapping)
 
         nics = []
-        if parsed_args.nic in ('auto', 'none'):
-            nics = [parsed_args.nic]
-        else:
-            for nic_str in parsed_args.nic:
+        auto_or_none = False
+        if parsed_args.nic is None:
+            parsed_args.nic = []
+        for nic_str in parsed_args.nic:
+            # Handle the special auto/none cases
+            if nic_str in ('auto', 'none'):
+                auto_or_none = True
+                nics.append(nic_str)
+            else:
                 nic_info = {"net-id": "", "v4-fixed-ip": "",
                             "v6-fixed-ip": "", "port-id": ""}
-                nic_info.update(dict(kv_str.split("=", 1)
-                                for kv_str in nic_str.split(",")))
+                for kv_str in nic_str.split(","):
+                    k, sep, v = kv_str.partition("=")
+                    if k in nic_info and v:
+                        nic_info[k] = v
+                    else:
+                        msg = (_("Invalid nic argument '%s'. Nic arguments "
+                                 "must be of the form --nic <net-id=net-uuid"
+                                 ",v4-fixed-ip=ip-addr,v6-fixed-ip=ip-addr,"
+                                 "port-id=port-uuid>."))
+                        raise exceptions.CommandError(msg % k)
                 if bool(nic_info["net-id"]) == bool(nic_info["port-id"]):
-                    msg = _("either net-id or port-id should be specified "
+                    msg = _("either network or port should be specified "
                             "but not both")
                     raise exceptions.CommandError(msg)
                 if self.app.client_manager.is_network_endpoint_enabled():
@@ -571,16 +685,44 @@ class CreateServer(command.ShowOne):
                         nic_info["port-id"] = port.id
                 else:
                     if nic_info["net-id"]:
-                        nic_info["net-id"] = utils.find_resource(
-                            compute_client.networks,
+                        nic_info["net-id"] = compute_client.api.network_find(
                             nic_info["net-id"]
-                        ).id
+                        )['id']
                     if nic_info["port-id"]:
                         msg = _("can't create server with port specified "
                                 "since network endpoint not enabled")
                         raise exceptions.CommandError(msg)
                 nics.append(nic_info)
 
+        if nics:
+            if auto_or_none:
+                if len(nics) > 1:
+                    msg = _('Specifying a --nic of auto or none cannot '
+                            'be used with any other --nic, --network '
+                            'or --port value.')
+                    raise exceptions.CommandError(msg)
+                nics = nics[0]
+        else:
+            # Default to empty list if nothing was specified, let nova side to
+            # decide the default behavior.
+            nics = []
+
+        # Check security group exist and convert ID to name
+        security_group_names = []
+        if self.app.client_manager.is_network_endpoint_enabled():
+            network_client = self.app.client_manager.network
+            for each_sg in parsed_args.security_group:
+                sg = network_client.find_security_group(each_sg,
+                                                        ignore_missing=False)
+                # Use security group ID to avoid multiple security group have
+                # same name in neutron networking backend
+                security_group_names.append(sg.id)
+        else:
+            # Handle nova-network case
+            for each_sg in parsed_args.security_group:
+                sg = compute_client.api.security_group_find(each_sg)
+                security_group_names.append(sg['name'])
+
         hints = {}
         for hint in parsed_args.hint:
             key, _sep, value = hint.partition('=')
@@ -610,7 +752,7 @@ class CreateServer(command.ShowOne):
             reservation_id=None,
             min_count=parsed_args.min,
             max_count=parsed_args.max,
-            security_groups=parsed_args.security_group,
+            security_groups=security_group_names,
             userdata=userdata,
             key_name=parsed_args.key_name,
             availability_zone=parsed_args.availability_zone,
@@ -791,23 +933,29 @@ class ListServer(command.Lister):
             default=False,
             help=_('List additional fields in output'),
         )
+        parser.add_argument(
+            '-n', '--no-name-lookup',
+            action='store_true',
+            default=False,
+            help=_('Skip flavor and image name lookup.'),
+        )
         parser.add_argument(
             '--marker',
-            metavar='<marker>',
+            metavar='<server>',
             default=None,
-            help=_('The last server (name or ID) of the previous page. Display'
-                   ' list of servers after marker. Display all servers if not'
-                   ' specified.')
+            help=_('The last server of the previous page. Display '
+                   'list of servers after marker. Display all servers if not '
+                   'specified. (name or ID)')
         )
         parser.add_argument(
             '--limit',
-            metavar='<limit>',
+            metavar='<num-servers>',
             type=int,
             default=None,
-            help=_("Maximum number of servers to display. If limit equals -1,"
-                   " all servers will be displayed. If limit is greater than"
-                   " 'osapi_max_limit' option of Nova API,"
-                   " 'osapi_max_limit' will be used instead."),
+            help=_("Maximum number of servers to display. If limit equals -1, "
+                   "all servers will be displayed. If limit is greater than "
+                   "'osapi_max_limit' option of Nova API, "
+                   "'osapi_max_limit' will be used instead."),
         )
         parser.add_argument(
             '--deleted',
@@ -897,6 +1045,8 @@ class ListServer(command.Lister):
                 'Networks',
                 'Image Name',
                 'Image ID',
+                'Flavor Name',
+                'Flavor ID',
                 'OS-EXT-AZ:availability_zone',
                 'OS-EXT-SRV-ATTR:host',
                 'Metadata',
@@ -910,6 +1060,8 @@ class ListServer(command.Lister):
                 'Networks',
                 'Image Name',
                 'Image ID',
+                'Flavor Name',
+                'Flavor ID',
                 'Availability Zone',
                 'Host',
                 'Properties',
@@ -921,19 +1073,31 @@ class ListServer(command.Lister):
                 'OS-EXT-SRV-ATTR:host',
             ]
         else:
-            columns = (
-                'ID',
-                'Name',
-                'Status',
-                'Networks',
-                'Image Name',
-            )
+            if parsed_args.no_name_lookup:
+                columns = (
+                    'ID',
+                    'Name',
+                    'Status',
+                    'Networks',
+                    'Image ID',
+                    'Flavor ID',
+                )
+            else:
+                columns = (
+                    'ID',
+                    'Name',
+                    'Status',
+                    'Networks',
+                    'Image Name',
+                    'Flavor Name',
+                )
             column_headers = (
                 'ID',
                 'Name',
                 'Status',
                 'Networks',
-                'Image Name',
+                'Image',
+                'Flavor',
             )
             mixed_case_fields = []
 
@@ -950,15 +1114,28 @@ class ListServer(command.Lister):
         # Create a dict that maps image_id to image object.
         # Needed so that we can display the "Image Name" column.
         # "Image Name" is not crucial, so we swallow any exceptions.
-        try:
-            images_list = self.app.client_manager.image.images.list()
-            for i in images_list:
-                images[i.id] = i
-        except Exception:
-            pass
+        if not parsed_args.no_name_lookup:
+            try:
+                images_list = self.app.client_manager.image.images.list()
+                for i in images_list:
+                    images[i.id] = i
+            except Exception:
+                pass
 
-        # Populate image_name and image_id attributes of server objects
-        # so that we can display "Image Name" and "Image ID" columns.
+        flavors = {}
+        # Create a dict that maps flavor_id to flavor object.
+        # Needed so that we can display the "Flavor Name" column.
+        # "Flavor Name" is not crucial, so we swallow any exceptions.
+        if not parsed_args.no_name_lookup:
+            try:
+                flavors_list = compute_client.flavors.list()
+                for i in flavors_list:
+                    flavors[i.id] = i
+            except Exception:
+                pass
+
+        # Populate image_name, image_id, flavor_name and flavor_id attributes
+        # of server objects so that we can display those columns.
         for s in data:
             if 'id' in s.image:
                 image = images.get(s.image['id'])
@@ -968,6 +1145,14 @@ class ListServer(command.Lister):
             else:
                 s.image_name = ''
                 s.image_id = ''
+            if 'id' in s.flavor:
+                flavor = flavors.get(s.flavor['id'])
+                if flavor:
+                    s.flavor_name = flavor.name
+                s.flavor_id = s.flavor['id']
+            else:
+                s.flavor_name = ''
+                s.flavor_id = ''
 
         table = (column_headers,
                  (utils.get_item_properties(
@@ -1084,6 +1269,10 @@ class MigrateServer(command.Command):
                 disk_over_commit=parsed_args.disk_overcommit,
             )
         else:
+            if parsed_args.block_migration or parsed_args.disk_overcommit:
+                raise exceptions.CommandError("--live must be specified if "
+                                              "--block-migration or "
+                                              "--disk-overcommit is specified")
             server.migrate()
 
         if parsed_args.wait:
@@ -1241,13 +1430,12 @@ class RemoveFixedIP(command.Command):
         parser.add_argument(
             "server",
             metavar="<server>",
-            help=_("Server (name or ID) to remove the fixed IP address from"),
+            help=_("Server to remove the fixed IP address from (name or ID)"),
         )
         parser.add_argument(
             "ip_address",
             metavar="<ip-address>",
-            help=_("Fixed IP address (IP address only) to remove from the "
-                   "server"),
+            help=_("Fixed IP address to remove from the server (IP only)"),
         )
         return parser
 
@@ -1268,14 +1456,14 @@ class RemoveFloatingIP(command.Command):
         parser.add_argument(
             "server",
             metavar="<server>",
-            help=_("Server (name or ID) to remove the "
-                   "floating IP address from"),
+            help=_(
+                "Server to remove the floating IP address from (name or ID)"
+            ),
         )
         parser.add_argument(
             "ip_address",
             metavar="<ip-address>",
-            help=_("Floating IP address (IP address only) "
-                   "to remove from server"),
+            help=_("Floating IP address to remove from server (IP only)"),
         )
         return parser
 
@@ -1288,6 +1476,39 @@ class RemoveFloatingIP(command.Command):
         server.remove_floating_ip(parsed_args.ip_address)
 
 
+class RemovePort(command.Command):
+    _description = _("Remove port from server")
+
+    def get_parser(self, prog_name):
+        parser = super(RemovePort, self).get_parser(prog_name)
+        parser.add_argument(
+            "server",
+            metavar="<server>",
+            help=_("Server to remove the port from (name or ID)"),
+        )
+        parser.add_argument(
+            "port",
+            metavar="<port>",
+            help=_("Port to remove from the server (name or ID)"),
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        compute_client = self.app.client_manager.compute
+
+        server = utils.find_resource(
+            compute_client.servers, parsed_args.server)
+
+        if self.app.client_manager.is_network_endpoint_enabled():
+            network_client = self.app.client_manager.network
+            port_id = network_client.find_port(
+                parsed_args.port, ignore_missing=False).id
+        else:
+            port_id = parsed_args.port
+
+        server.interface_detach(port_id)
+
+
 class RemoveServerSecurityGroup(command.Command):
     _description = _("Remove security group from server")
 
@@ -1312,12 +1533,11 @@ class RemoveServerSecurityGroup(command.Command):
             compute_client.servers,
             parsed_args.server,
         )
-        security_group = utils.find_resource(
-            compute_client.security_groups,
+        security_group = compute_client.api.security_group_find(
             parsed_args.group,
         )
 
-        server.remove_security_group(security_group.id)
+        server.remove_security_group(security_group['id'])
 
 
 class RemoveServerVolume(command.Command):
@@ -1379,7 +1599,13 @@ class RescueServer(command.ShowOne):
 
 
 class ResizeServer(command.Command):
-    _description = _("Scale server to a new flavor")
+    _description = _("""Scale server to a new flavor.
+
+A resize operation is implemented by creating a new server and copying the
+contents of the original disk into a new one. It is also a two-step process for
+the user: the first is to perform the resize, the second is to either confirm
+(verify) success and release the old server, or to declare a revert to release
+the new server and restart the old one.""")
 
     def get_parser(self, prog_name):
         parser = super(ResizeServer, self).get_parser(prog_name)
