@@ -55,7 +55,7 @@ def _determine_ec2_user(parsed_args, client_manager):
 
 
 class CreateEC2Creds(command.ShowOne):
-    """Create EC2 credentials"""
+    _description = _("Create EC2 credentials")
 
     def get_parser(self, prog_name):
         parser = super(CreateEC2Creds, self).get_parser(prog_name)
@@ -112,7 +112,7 @@ class CreateEC2Creds(command.ShowOne):
 
 
 class DeleteEC2Creds(command.Command):
-    """Delete EC2 credentials"""
+    _description = _("Delete EC2 credentials")
 
     def get_parser(self, prog_name):
         parser = super(DeleteEC2Creds, self).get_parser(prog_name)
@@ -140,8 +140,8 @@ class DeleteEC2Creds(command.Command):
             except Exception as e:
                 result += 1
                 LOG.error(_("Failed to delete EC2 credentials with "
-                          "access key '%(access_key)s': %(e)s")
-                          % {'access_key': i, 'e': e})
+                          "access key '%(access_key)s': %(e)s"),
+                          {'access_key': i, 'e': e})
 
         if result > 0:
             total = len(parsed_args.access_key)
@@ -151,7 +151,7 @@ class DeleteEC2Creds(command.Command):
 
 
 class ListEC2Creds(command.Lister):
-    """List EC2 credentials"""
+    _description = _("List EC2 credentials")
 
     def get_parser(self, prog_name):
         parser = super(ListEC2Creds, self).get_parser(prog_name)
@@ -179,7 +179,7 @@ class ListEC2Creds(command.Lister):
 
 
 class ShowEC2Creds(command.ShowOne):
-    """Display EC2 credentials details"""
+    _description = _("Display EC2 credentials details")
 
     def get_parser(self, prog_name):
         parser = super(ShowEC2Creds, self).get_parser(prog_name)
