@@ -36,8 +36,11 @@ from openstackclient.api import utils as api_utils
 from openstackclient.i18n import _
 
 
+CONTAINER_CHOICES = ["ami", "ari", "aki", "bare", "docker", "ova", "ovf"]
 DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
+DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
+                "vdi", "iso", "ploop"]
 
 
 LOG = logging.getLogger(__name__)
@@ -59,7 +62,7 @@ def _format_visibility(data):
 
 
 class CreateImage(command.ShowOne):
-    """Create/upload an image"""
+    _description = _("Create/upload an image")
 
     def get_parser(self, prog_name):
         parser = super(CreateImage, self).get_parser(prog_name)
@@ -82,15 +85,20 @@ class CreateImage(command.ShowOne):
             "--container-format",
             default=DEFAULT_CONTAINER_FORMAT,
             metavar="<container-format>",
-            help=_("Image container format "
-                   "(default: %s)") % DEFAULT_CONTAINER_FORMAT,
+            choices=CONTAINER_CHOICES,
+            help=(_("Image container format. "
+                    "The supported options are: %(option_list)s. "
+                    "The default format is: %(default_opt)s") %
+                  {'option_list': ', '.join(CONTAINER_CHOICES),
+                   'default_opt': DEFAULT_CONTAINER_FORMAT})
         )
         parser.add_argument(
             "--disk-format",
             default=DEFAULT_DISK_FORMAT,
             metavar="<disk-format>",
-            help=_("Image disk format "
-                   "(default: %s)") % DEFAULT_DISK_FORMAT,
+            choices=DISK_CHOICES,
+            help=_("Image disk format. The supported options are: %s. "
+                   "The default format is: raw") % ', '.join(DISK_CHOICES)
         )
         parser.add_argument(
             "--size",
@@ -120,12 +128,13 @@ class CreateImage(command.ShowOne):
             metavar="<image-url>",
             help=_("Copy image from the data store (similar to --location)"),
         )
-        parser.add_argument(
+        source_group = parser.add_mutually_exclusive_group()
+        source_group.add_argument(
             "--file",
             metavar="<file>",
             help=_("Upload image from local file"),
         )
-        parser.add_argument(
+        source_group.add_argument(
             "--volume",
             metavar="<volume>",
             help=_("Create image from a volume"),
@@ -277,7 +286,7 @@ class CreateImage(command.ShowOne):
 
 
 class DeleteImage(command.Command):
-    """Delete image(s)"""
+    _description = _("Delete image(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteImage, self).get_parser(prog_name)
@@ -300,7 +309,7 @@ class DeleteImage(command.Command):
 
 
 class ListImage(command.Lister):
-    """List available images"""
+    _description = _("List available images")
 
     def get_parser(self, prog_name):
         parser = super(ListImage, self).get_parser(prog_name)
@@ -350,8 +359,9 @@ class ListImage(command.Lister):
         parser.add_argument(
             '--sort',
             metavar="<key>[:<direction>]",
+            default='name:asc',
             help=_("Sort output by selected keys and directions(asc or desc) "
-                   "(default: asc), multiple keys and directions can be "
+                   "(default: name:asc), multiple keys and directions can be "
                    "specified separated by comma"),
         )
         return parser
@@ -440,7 +450,7 @@ class ListImage(command.Lister):
 
 
 class SaveImage(command.Command):
-    """Save an image locally"""
+    _description = _("Save an image locally")
 
     def get_parser(self, prog_name):
         parser = super(SaveImage, self).get_parser(prog_name)
@@ -468,7 +478,7 @@ class SaveImage(command.Command):
 
 
 class SetImage(command.Command):
-    """Set image properties"""
+    _description = _("Set image properties")
 
     def get_parser(self, prog_name):
         parser = super(SetImage, self).get_parser(prog_name)
@@ -494,22 +504,19 @@ class SetImage(command.Command):
             type=int,
             help=_("Minimum RAM size needed to boot image, in megabytes"),
         )
-        container_choices = ["ami", "ari", "aki", "bare", "ovf"]
         parser.add_argument(
             "--container-format",
             metavar="<container-format>",
-            help=_("Container format of image. Acceptable formats: %s") %
-            container_choices,
-            choices=container_choices
+            choices=CONTAINER_CHOICES,
+            help=_("Image container format. The supported options are: %s") %
+            ', '.join(CONTAINER_CHOICES)
         )
-        disk_choices = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2",
-                        "vdi", "iso"]
         parser.add_argument(
             "--disk-format",
             metavar="<disk-format>",
-            help=_("Disk format of image. Acceptable formats: %s") %
-            disk_choices,
-            choices=disk_choices
+            choices=DISK_CHOICES,
+            help=_("Image disk format. The supported options are: %s.") %
+            ', '.join(DISK_CHOICES)
         )
         parser.add_argument(
             "--size",
@@ -702,10 +709,16 @@ class SetImage(command.Command):
 
 
 class ShowImage(command.ShowOne):
-    """Display image details"""
+    _description = _("Display image details")
 
     def get_parser(self, prog_name):
         parser = super(ShowImage, self).get_parser(prog_name)
+        parser.add_argument(
+            "--human-readable",
+            default=False,
+            action='store_true',
+            help=_("Print image size in a human-friendly format."),
+        )
         parser.add_argument(
             "image",
             metavar="<image>",
@@ -722,5 +735,8 @@ class ShowImage(command.ShowOne):
 
         info = {}
         info.update(image._info)
+        if parsed_args.human_readable:
+            if 'size' in info:
+                info['size'] = utils.format_size(info['size'])
         info['properties'] = utils.format_dict(info.get('properties', {}))
         return zip(*sorted(six.iteritems(info)))
