@@ -20,13 +20,18 @@ from osc_lib import exceptions
 from osc_lib import utils
 
 from openstackclient.i18n import _
+from openstackclient.network import sdk_utils
 
 
 LOG = logging.getLogger(__name__)
 
 
+def _get_columns(item):
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, {})
+
+
 class CreateNetworkSegment(command.ShowOne):
-    """Create new network segment"""
+    _description = _("Create new network segment")
 
     def get_parser(self, prog_name):
         parser = super(CreateNetworkSegment, self).get_parser(prog_name)
@@ -84,13 +89,13 @@ class CreateNetworkSegment(command.ShowOne):
         if parsed_args.segment is not None:
             attrs['segmentation_id'] = parsed_args.segment
         obj = client.create_segment(**attrs)
-        columns = tuple(sorted(obj.keys()))
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns)
-        return (columns, data)
+        return (display_columns, data)
 
 
 class DeleteNetworkSegment(command.Command):
-    """Delete network segment(s)"""
+    _description = _("Delete network segment(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteNetworkSegment, self).get_parser(prog_name)
@@ -114,8 +119,8 @@ class DeleteNetworkSegment(command.Command):
             except Exception as e:
                 result += 1
                 LOG.error(_("Failed to delete network segment with "
-                            "ID '%(network_segment)s': %(e)s")
-                          % {'network_segment': network_segment, 'e': e})
+                            "ID '%(network_segment)s': %(e)s"),
+                          {'network_segment': network_segment, 'e': e})
 
         if result > 0:
             total = len(parsed_args.network_segment)
@@ -125,7 +130,7 @@ class DeleteNetworkSegment(command.Command):
 
 
 class ListNetworkSegment(command.Lister):
-    """List network segments"""
+    _description = _("List network segments")
 
     def get_parser(self, prog_name):
         parser = super(ListNetworkSegment, self).get_parser(prog_name)
@@ -185,7 +190,7 @@ class ListNetworkSegment(command.Lister):
 
 
 class SetNetworkSegment(command.Command):
-    """Set network segment properties"""
+    _description = _("Set network segment properties")
 
     def get_parser(self, prog_name):
         parser = super(SetNetworkSegment, self).get_parser(prog_name)
@@ -219,7 +224,7 @@ class SetNetworkSegment(command.Command):
 
 
 class ShowNetworkSegment(command.ShowOne):
-    """Display network segment details"""
+    _description = _("Display network segment details")
 
     def get_parser(self, prog_name):
         parser = super(ShowNetworkSegment, self).get_parser(prog_name)
@@ -236,6 +241,6 @@ class ShowNetworkSegment(command.ShowOne):
             parsed_args.network_segment,
             ignore_missing=False
         )
-        columns = tuple(sorted(obj.keys()))
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns)
-        return (columns, data)
+        return (display_columns, data)
