@@ -16,8 +16,11 @@ import uuid
 from openstackclient.tests.functional.network.v2 import common
 
 
-class PortTests(common.NetworkTests):
+class PortTests(common.NetworkTagTests):
     """Functional tests for port"""
+
+    base_command = 'port'
+
     NAME = uuid.uuid4().hex
     NETWORK_NAME = uuid.uuid4().hex
 
@@ -25,18 +28,24 @@ class PortTests(common.NetworkTests):
     def setUpClass(cls):
         common.NetworkTests.setUpClass()
         if cls.haz_network:
-            # Create a network for the port
+            cls.NAME = uuid.uuid4().hex
+            cls.NETWORK_NAME = uuid.uuid4().hex
+
+            # Create a network for the port tests
             cls.openstack(
                 'network create ' + cls.NETWORK_NAME
             )
 
     @classmethod
     def tearDownClass(cls):
-        if cls.haz_network:
-            raw_output = cls.openstack(
-                'network delete ' + cls.NETWORK_NAME
-            )
-            cls.assertOutput('', raw_output)
+        try:
+            if cls.haz_network:
+                raw_output = cls.openstack(
+                    'network delete ' + cls.NETWORK_NAME
+                )
+                cls.assertOutput('', raw_output)
+        finally:
+            super(PortTests, cls).tearDownClass()
 
     def setUp(self):
         super(PortTests, self).setUp()
@@ -246,3 +255,9 @@ class PortTests(common.NetworkTests):
             sg_id2,
             json_output.get('security_group_ids'),
         )
+
+    def _create_resource_for_tag_test(self, name, args):
+        return json.loads(self.openstack(
+            '{} create -f json --network {} {} {}'
+            .format(self.base_command, self.NETWORK_NAME, args, name)
+        ))
