@@ -144,7 +144,7 @@ class FakeAddressScope(object):
         If address scopes list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List address scopes:
+        :param List address_scopes:
             A list of FakeResource objects faking address scopes
         :param int count:
             The number of address scopes to fake
@@ -346,9 +346,11 @@ class FakeNetwork(object):
             'availability_zone_hints': [],
             'is_default': False,
             'port_security_enabled': True,
+            'tags': ['test'],
             'qos_policy_id': 'qos-policy-id-' + uuid.uuid4().hex,
             'ipv4_address_scope': 'ipv4' + uuid.uuid4().hex,
             'ipv6_address_scope': 'ipv6' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -365,6 +367,7 @@ class FakeNetwork(object):
             network_attrs['port_security_enabled']
         network.subnet_ids = network_attrs['subnets']
         network.is_shared = network_attrs['shared']
+        network.is_tags = network_attrs['tags']
         network.provider_network_type = \
             network_attrs['provider:network_type']
         network.provider_physical_network = \
@@ -556,6 +559,7 @@ class FakePort(object):
             'binding:vif_details': {},
             'binding:vif_type': 'ovs',
             'binding:vnic_type': 'normal',
+            'data_plane_status': None,
             'description': 'description-' + uuid.uuid4().hex,
             'device_id': 'device-id-' + uuid.uuid4().hex,
             'device_owner': 'compute:nova',
@@ -572,6 +576,8 @@ class FakePort(object):
             'security_group_ids': [],
             'status': 'ACTIVE',
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
+            'qos_policy_id': 'qos-policy-id-' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -590,6 +596,7 @@ class FakePort(object):
         port.is_port_security_enabled = port_attrs['port_security_enabled']
         port.project_id = port_attrs['tenant_id']
         port.security_group_ids = port_attrs['security_group_ids']
+        port.qos_policy_id = port_attrs['qos_policy_id']
 
         return port
 
@@ -769,6 +776,53 @@ class FakeNetworkRBAC(object):
         return mock.Mock(side_effect=rbac_policies)
 
 
+class FakeNetworkFlavorProfile(object):
+    """Fake network flavor profile."""
+
+    @staticmethod
+    def create_one_service_profile(attrs=None):
+        """Create flavor profile."""
+        attrs = attrs or {}
+
+        flavor_profile_attrs = {
+            'id': 'flavor-profile-id' + uuid.uuid4().hex,
+            'description': 'flavor-profile-description-' + uuid.uuid4().hex,
+            'tenant_id': 'project-id-' + uuid.uuid4().hex,
+            'driver': 'driver-' + uuid.uuid4().hex,
+            'metainfo': 'metainfo-' + uuid.uuid4().hex,
+            'enabled': True
+        }
+
+        flavor_profile_attrs.update(attrs)
+
+        flavor_profile = fakes.FakeResource(
+            info=copy.deepcopy(flavor_profile_attrs),
+            loaded=True)
+
+        flavor_profile.project_id = flavor_profile_attrs['tenant_id']
+        flavor_profile.is_enabled = flavor_profile_attrs['enabled']
+
+        return flavor_profile
+
+    @staticmethod
+    def create_service_profile(attrs=None, count=2):
+        """Create multiple flavor profiles."""
+
+        flavor_profiles = []
+        for i in range(0, count):
+            flavor_profiles.append(FakeNetworkFlavorProfile.
+                                   create_one_service_profile(attrs))
+        return flavor_profiles
+
+    @staticmethod
+    def get_service_profile(flavor_profile=None, count=2):
+        """Get a list of flavor profiles."""
+        if flavor_profile is None:
+            flavor_profile = (FakeNetworkFlavorProfile.
+                              create_service_profile(count))
+        return mock.Mock(side_effect=flavor_profile)
+
+
 class FakeNetworkQosPolicy(object):
     """Fake one or more QoS policies."""
 
@@ -790,6 +844,7 @@ class FakeNetworkQosPolicy(object):
         qos_policy_attrs = {
             'name': 'qos-policy-name-' + uuid.uuid4().hex,
             'id': qos_id,
+            'is_default': False,
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
             'shared': False,
             'description': 'qos-policy-description-' + uuid.uuid4().hex,
@@ -834,7 +889,7 @@ class FakeNetworkQosPolicy(object):
         If qos policies list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List address scopes:
+        :param List qos_policies:
             A list of FakeResource objects faking qos policies
         :param int count:
             The number of QoS policies to fake
@@ -872,6 +927,7 @@ class FakeNetworkQosRule(object):
         if type == RULE_TYPE_BANDWIDTH_LIMIT:
             qos_rule_attrs['max_kbps'] = randint(1, 10000)
             qos_rule_attrs['max_burst_kbits'] = randint(1, 10000)
+            qos_rule_attrs['direction'] = 'egress'
         elif type == RULE_TYPE_DSCP_MARKING:
             qos_rule_attrs['dscp_mark'] = choice(VALID_DSCP_MARKS)
         elif type == RULE_TYPE_MINIMUM_BANDWIDTH:
@@ -912,7 +968,7 @@ class FakeNetworkQosRule(object):
         If Network QoS rules list is provided, then initialize the Mock
         object with the list. Otherwise create one.
 
-        :param List address scopes:
+        :param List qos_rules:
             A list of FakeResource objects faking Network QoS rules
         :param int count:
             The number of QoS minimum bandwidth rules to fake
@@ -999,6 +1055,7 @@ class FakeRouter(object):
             'external_gateway_info': {},
             'availability_zone_hints': [],
             'availability_zones': [],
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -1071,7 +1128,7 @@ class FakeSecurityGroup(object):
             'id': 'security-group-id-' + uuid.uuid4().hex,
             'name': 'security-group-name-' + uuid.uuid4().hex,
             'description': 'security-group-description-' + uuid.uuid4().hex,
-            'tenant_id': 'project-id-' + uuid.uuid4().hex,
+            'project_id': 'project-id-' + uuid.uuid4().hex,
             'security_group_rules': [],
         }
 
@@ -1083,7 +1140,7 @@ class FakeSecurityGroup(object):
             loaded=True)
 
         # Set attributes with special mapping in OpenStack SDK.
-        security_group.project_id = security_group_attrs['tenant_id']
+        security_group.project_id = security_group_attrs['project_id']
 
         return security_group
 
@@ -1112,7 +1169,7 @@ class FakeSecurityGroup(object):
         If security groups list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List security groups:
+        :param List security_groups:
             A list of FakeResource objects faking security groups
         :param int count:
             The number of security groups to fake
@@ -1192,7 +1249,7 @@ class FakeSecurityGroupRule(object):
         If security group rules list is provided, then initialize the Mock
         object with the list. Otherwise create one.
 
-        :param List security group rules:
+        :param List security_group_rules:
             A list of FakeResource objects faking security group rules
         :param int count:
             The number of security group rules to fake
@@ -1240,6 +1297,7 @@ class FakeSubnet(object):
             'service_types': [],
             'subnetpool_id': None,
             'description': 'subnet-description-' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -1357,7 +1415,7 @@ class FakeFloatingIP(object):
         If floating_ips list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List floating ips:
+        :param List floating_ips:
             A list of FakeResource objects faking floating ips
         :param int count:
             The number of floating ips to fake
@@ -1490,6 +1548,7 @@ class FakeSubnetPool(object):
             'default_quota': None,
             'ip_version': '4',
             'description': 'subnet-pool-description-' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -1536,7 +1595,7 @@ class FakeSubnetPool(object):
         If subnet_pools list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List subnet pools:
+        :param List subnet_pools:
             A list of FakeResource objects faking subnet pools
         :param int count:
             The number of subnet pools to fake
@@ -1582,3 +1641,53 @@ class FakeNetworkServiceProvider(object):
                                      create_one_network_service_provider(
                                          attrs))
         return service_providers
+
+
+class FakeQuota(object):
+    """Fake quota"""
+
+    @staticmethod
+    def create_one_net_quota(attrs=None):
+        """Create one quota"""
+        attrs = attrs or {}
+
+        quota_attrs = {
+            'floating_ips': 20,
+            'networks': 25,
+            'ports': 11,
+            'rbac_policies': 15,
+            'routers': 40,
+            'security_groups': 10,
+            'security_group_rules': 100,
+            'subnets': 20,
+            'subnet_pools': 30}
+
+        quota_attrs.update(attrs)
+
+        quota = fakes.FakeResource(
+            info=copy.deepcopy(quota_attrs),
+            loaded=True)
+        return quota
+
+    @staticmethod
+    def create_one_default_net_quota(attrs=None):
+        """Create one quota"""
+        attrs = attrs or {}
+
+        quota_attrs = {
+            'floatingip': 30,
+            'network': 20,
+            'port': 10,
+            'rbac_policy': 25,
+            'router': 30,
+            'security_group': 30,
+            'security_group_rule': 200,
+            'subnet': 10,
+            'subnetpool': 20}
+
+        quota_attrs.update(attrs)
+
+        quota = fakes.FakeResource(
+            info=copy.deepcopy(quota_attrs),
+            loaded=True)
+        return quota
