@@ -35,7 +35,8 @@ class TestPort(network_fakes.TestNetworkV2):
         # Get a shortcut to the ProjectManager Mock
         self.projects_mock = self.app.client_manager.identity.projects
 
-    def _get_common_cols_data(self, fake_port):
+    @staticmethod
+    def _get_common_cols_data(fake_port):
         columns = (
             'admin_state_up',
             'allowed_address_pairs',
@@ -44,6 +45,7 @@ class TestPort(network_fakes.TestNetworkV2):
             'binding_vif_details',
             'binding_vif_type',
             'binding_vnic_type',
+            'data_plane_status',
             'description',
             'device_id',
             'device_owner',
@@ -60,6 +62,7 @@ class TestPort(network_fakes.TestNetworkV2):
             'qos_policy_id',
             'security_group_ids',
             'status',
+            'tags',
         )
 
         data = (
@@ -70,6 +73,7 @@ class TestPort(network_fakes.TestNetworkV2):
             utils.format_dict(fake_port.binding_vif_details),
             fake_port.binding_vif_type,
             fake_port.binding_vnic_type,
+            fake_port.data_plane_status,
             fake_port.description,
             fake_port.device_id,
             fake_port.device_owner,
@@ -86,6 +90,7 @@ class TestPort(network_fakes.TestNetworkV2):
             fake_port.qos_policy_id,
             utils.format_list(fake_port.security_group_ids),
             fake_port.status,
+            utils.format_list(fake_port.tags),
         )
 
         return columns, data
@@ -94,11 +99,13 @@ class TestPort(network_fakes.TestNetworkV2):
 class TestCreatePort(TestPort):
 
     _port = network_fakes.FakePort.create_one_port()
+    columns, data = TestPort._get_common_cols_data(_port)
 
     def setUp(self):
         super(TestCreatePort, self).setUp()
 
         self.network.create_port = mock.Mock(return_value=self._port)
+        self.network.set_tags = mock.Mock(return_value=None)
         fake_net = network_fakes.FakeNetwork.create_one_network({
             'id': self._port.network_id,
         })
@@ -127,10 +134,10 @@ class TestCreatePort(TestPort):
             'network_id': self._port.network_id,
             'name': 'test-port',
         })
+        self.assertFalse(self.network.set_tags.called)
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_full_options(self):
         arglist = [
@@ -164,7 +171,6 @@ class TestCreatePort(TestPort):
             ('network', self._port.network_id),
             ('dns_name', '8.8.8.8'),
             ('name', 'test-port'),
-
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -185,9 +191,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_invalid_json_binding_profile(self):
         arglist = [
@@ -237,9 +242,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_with_security_group(self):
         secgroup = network_fakes.FakeSecurityGroup.create_one_security_group()
@@ -267,9 +271,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_with_dns_name(self):
         arglist = [
@@ -294,9 +297,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_with_security_groups(self):
         sg_1 = network_fakes.FakeSecurityGroup.create_one_security_group()
@@ -325,9 +327,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_with_no_security_groups(self):
         arglist = [
@@ -352,9 +353,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_with_allowed_address_pair_ipaddr(self):
         pairs = [{'ip_address': '192.168.1.123'},
@@ -383,9 +383,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_with_allowed_address_pair(self):
         pairs = [{'ip_address': '192.168.1.123',
@@ -420,9 +419,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_with_qos(self):
         qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
@@ -449,9 +447,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_security_enabled(self):
         arglist = [
@@ -501,6 +498,48 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
+    def _test_create_with_tag(self, add_tags=True):
+        arglist = [
+            '--network', self._port.network_id,
+            'test-port',
+        ]
+        if add_tags:
+            arglist += ['--tag', 'red', '--tag', 'blue']
+        else:
+            arglist += ['--no-tag']
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('enable', True),
+            ('name', 'test-port'),
+        ]
+        if add_tags:
+            verifylist.append(('tags', ['red', 'blue']))
+        else:
+            verifylist.append(('no_tag', True))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(
+            admin_state_up=True,
+            network_id=self._port.network_id,
+            name='test-port'
+        )
+        if add_tags:
+            self.network.set_tags.assert_called_once_with(
+                self._port,
+                tests_utils.CompareBySet(['red', 'blue']))
+        else:
+            self.assertFalse(self.network.set_tags.called)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def test_create_with_tags(self):
+        self._test_create_with_tag(add_tags=True)
+
+    def test_create_with_no_tag(self):
+        self._test_create_with_tag(add_tags=False)
+
 
 class TestDeletePort(TestPort):
 
@@ -601,6 +640,7 @@ class TestListPort(TestPort):
         'Status',
         'Security Groups',
         'Device Owner',
+        'Tags',
     )
 
     data = []
@@ -623,6 +663,7 @@ class TestListPort(TestPort):
             prt.status,
             utils.format_list(prt.security_group_ids),
             prt.device_owner,
+            utils.format_list(prt.tags),
         ))
 
     def setUp(self):
@@ -899,10 +940,35 @@ class TestListPort(TestPort):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
+    def test_list_with_tag_options(self):
+        arglist = [
+            '--tags', 'red,blue',
+            '--any-tags', 'red,green',
+            '--not-tags', 'orange,yellow',
+            '--not-any-tags', 'black,white',
+        ]
+        verifylist = [
+            ('tags', ['red', 'blue']),
+            ('any_tags', ['red', 'green']),
+            ('not_tags', ['orange', 'yellow']),
+            ('not_any_tags', ['black', 'white']),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.ports.assert_called_once_with(
+            **{'tags': 'red,blue',
+               'any_tags': 'red,green',
+               'not_tags': 'orange,yellow',
+               'not_any_tags': 'black,white'}
+        )
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
 
 class TestSetPort(TestPort):
 
-    _port = network_fakes.FakePort.create_one_port()
+    _port = network_fakes.FakePort.create_one_port({'tags': ['green', 'red']})
 
     def setUp(self):
         super(TestSetPort, self).setUp()
@@ -910,6 +976,7 @@ class TestSetPort(TestPort):
         self.network.find_subnet = mock.Mock(return_value=self.fake_subnet)
         self.network.find_port = mock.Mock(return_value=self._port)
         self.network.update_port = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
 
         # Get the command object to test
         self.cmd = port.SetPort(self.app, self.namespace)
@@ -924,8 +991,8 @@ class TestSetPort(TestPort):
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.take_action(parsed_args)
-        attrs = {}
-        self.network.update_port.assert_called_once_with(self._port, **attrs)
+        self.assertFalse(self.network.update_port.called)
+        self.assertFalse(self.network.set_tags.called)
         self.assertIsNone(result)
 
     def test_set_port_fixed_ip(self):
@@ -1371,11 +1438,74 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
 
+    def test_set_port_data_plane_status(self):
+        _testport = network_fakes.FakePort.create_one_port(
+            {'data_plane_status': None})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        arglist = [
+            '--data-plane-status', 'ACTIVE',
+            _testport.name,
+        ]
+        verifylist = [
+            ('data_plane_status', 'ACTIVE'),
+            ('port', _testport.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'data_plane_status': 'ACTIVE',
+        }
+
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_set_port_invalid_data_plane_status_value(self):
+        arglist = [
+            '--data-plane-status', 'Spider-Man',
+            'test-port',
+        ]
+        self.assertRaises(tests_utils.ParserException,
+                          self.check_parser,
+                          self.cmd,
+                          arglist,
+                          None)
+
+    def _test_set_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['red', 'blue', 'green']
+        else:
+            arglist = ['--no-tag']
+            verifylist = [('no_tag', True)]
+            expected_args = []
+        arglist.append(self._port.name)
+        verifylist.append(
+            ('port', self._port.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_port.called)
+        self.network.set_tags.assert_called_once_with(
+            self._port,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_set_with_tags(self):
+        self._test_set_tags(with_tags=True)
+
+    def test_set_with_no_tag(self):
+        self._test_set_tags(with_tags=False)
+
 
 class TestShowPort(TestPort):
 
     # The port to show.
     _port = network_fakes.FakePort.create_one_port()
+    columns, data = TestPort._get_common_cols_data(_port)
 
     def setUp(self):
         super(TestShowPort, self).setUp()
@@ -1406,9 +1536,8 @@ class TestShowPort(TestPort):
         self.network.find_port.assert_called_once_with(
             self._port.name, ignore_missing=False)
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
 
 class TestUnsetPort(TestPort):
@@ -1420,12 +1549,14 @@ class TestUnsetPort(TestPort):
                             'ip_address': '0.0.0.1'},
                            {'subnet_id': '042eb10a-3a18-4658-ab-cf47c8d03152',
                             'ip_address': '1.0.0.0'}],
-             'binding:profile': {'batman': 'Joker', 'Superman': 'LexLuthor'}})
+             'binding:profile': {'batman': 'Joker', 'Superman': 'LexLuthor'},
+             'tags': ['green', 'red'], })
         self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet(
             {'id': '042eb10a-3a18-4658-ab-cf47c8d03152'})
         self.network.find_subnet = mock.Mock(return_value=self.fake_subnet)
         self.network.find_port = mock.Mock(return_value=self._testport)
         self.network.update_port = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
         # Get the command object to test
         self.cmd = port.UnsetPort(self.app, self.namespace)
 
@@ -1573,3 +1704,54 @@ class TestUnsetPort(TestPort):
         self.assertRaises(exceptions.CommandError,
                           self.cmd.take_action,
                           parsed_args)
+
+    def test_unset_port_data_plane_status(self):
+        _fake_port = network_fakes.FakePort.create_one_port(
+            {'data_plane_status': 'ACTIVE'})
+        self.network.find_port = mock.Mock(return_value=_fake_port)
+        arglist = [
+            '--data-plane-status',
+            _fake_port.name,
+        ]
+        verifylist = [
+            ('data_plane_status', True),
+            ('port', _fake_port.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'data_plane_status': None,
+        }
+
+        self.network.update_port.assert_called_once_with(_fake_port, **attrs)
+        self.assertIsNone(result)
+
+    def _test_unset_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['green']
+        else:
+            arglist = ['--all-tag']
+            verifylist = [('all_tag', True)]
+            expected_args = []
+        arglist.append(self._testport.name)
+        verifylist.append(
+            ('port', self._testport.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_port.called)
+        self.network.set_tags.assert_called_once_with(
+            self._testport,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_unset_with_tags(self):
+        self._test_unset_tags(with_tags=True)
+
+    def test_unset_with_all_tag(self):
+        self._test_unset_tags(with_tags=False)
