@@ -68,8 +68,8 @@ class TestVolumeCreate(TestVolume):
         'bootable',
         'created_at',
         'display_description',
-        'display_name',
         'id',
+        'name',
         'properties',
         'size',
         'snapshot_id',
@@ -86,8 +86,8 @@ class TestVolumeCreate(TestVolume):
             self.new_volume.bootable,
             self.new_volume.created_at,
             self.new_volume.display_description,
-            self.new_volume.display_name,
             self.new_volume.id,
+            self.new_volume.display_name,
             utils.format_dict(self.new_volume.metadata),
             self.new_volume.size,
             self.new_volume.snapshot_id,
@@ -431,6 +431,142 @@ class TestVolumeCreate(TestVolume):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, data)
 
+    def test_volume_create_with_bootable_and_readonly(self):
+        arglist = [
+            '--bootable',
+            '--read-only',
+            '--size', str(self.new_volume.size),
+            self.new_volume.display_name,
+        ]
+        verifylist = [
+            ('bootable', True),
+            ('non_bootable', False),
+            ('read_only', True),
+            ('read_write', False),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.display_name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            self.new_volume.size,
+            None,
+            None,
+            self.new_volume.display_name,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, True)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, True)
+
+    def test_volume_create_with_nonbootable_and_readwrite(self):
+        arglist = [
+            '--non-bootable',
+            '--read-write',
+            '--size', str(self.new_volume.size),
+            self.new_volume.display_name,
+        ]
+        verifylist = [
+            ('bootable', False),
+            ('non_bootable', True),
+            ('read_only', False),
+            ('read_write', True),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.display_name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            self.new_volume.size,
+            None,
+            None,
+            self.new_volume.display_name,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, False)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, False)
+
+    @mock.patch.object(volume.LOG, 'error')
+    def test_volume_create_with_bootable_and_readonly_fail(
+            self, mock_error):
+
+        self.volumes_mock.set_bootable.side_effect = (
+            exceptions.CommandError())
+
+        self.volumes_mock.update_readonly_flag.side_effect = (
+            exceptions.CommandError())
+
+        arglist = [
+            '--bootable',
+            '--read-only',
+            '--size', str(self.new_volume.size),
+            self.new_volume.display_name,
+        ]
+        verifylist = [
+            ('bootable', True),
+            ('non_bootable', False),
+            ('read_only', True),
+            ('read_write', False),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.display_name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            self.new_volume.size,
+            None,
+            None,
+            self.new_volume.display_name,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+        )
+
+        self.assertEqual(2, mock_error.call_count)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, True)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, True)
+
     def test_volume_create_without_size(self):
         arglist = [
             self.new_volume.display_name,
@@ -462,6 +598,38 @@ class TestVolumeCreate(TestVolume):
         self.assertRaises(tests_utils.ParserException, self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_volume_create_backward_compatibility(self):
+        arglist = [
+            '-c', 'display_name',
+            '--size', str(self.new_volume.size),
+            self.new_volume.display_name,
+        ]
+        verifylist = [
+            ('columns', ['display_name']),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.display_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            self.new_volume.size,
+            None,
+            None,
+            self.new_volume.display_name,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+        )
+        self.assertIn('display_name', columns)
+        self.assertNotIn('name', columns)
+        self.assertIn(self.new_volume.display_name, data)
+
 
 class TestVolumeDelete(TestVolume):
 
@@ -559,7 +727,7 @@ class TestVolumeList(TestVolume):
     _volume = volume_fakes.FakeVolume.create_one_volume()
     columns = (
         'ID',
-        'Display Name',
+        'Name',
         'Status',
         'Size',
         'Attached to',
@@ -670,7 +838,7 @@ class TestVolumeList(TestVolume):
 
         collist = (
             'ID',
-            'Display Name',
+            'Name',
             'Status',
             'Size',
             'Type',
@@ -727,6 +895,27 @@ class TestVolumeList(TestVolume):
         self.assertRaises(argparse.ArgumentTypeError, self.check_parser,
                           self.cmd, arglist, verifylist)
 
+    def test_volume_list_backward_compatibility(self):
+        arglist = [
+            '-c', 'Display Name',
+        ]
+        verifylist = [
+            ('columns', ['Display Name']),
+            ('long', False),
+            ('all_projects', False),
+            ('name', None),
+            ('status', None),
+            ('limit', None),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.assertIn('Display Name', columns)
+        self.assertNotIn('Name', columns)
+        for each_volume in data:
+            self.assertIn(self._volume.display_name, each_volume)
+
 
 class TestVolumeMigrate(TestVolume):
 
@@ -935,6 +1124,7 @@ class TestVolumeSet(TestVolume):
 
     def test_volume_set_property(self):
         arglist = [
+            '--no-property',
             '--property', 'myprop=myvalue',
             self._volume.display_name,
         ]
@@ -944,6 +1134,7 @@ class TestVolumeSet(TestVolume):
             ('name', None),
             ('description', None),
             ('size', None),
+            ('no_property', True),
             ('property', {'myprop': 'myvalue'}),
             ('volume', self._volume.display_name),
             ('bootable', False),
@@ -961,6 +1152,10 @@ class TestVolumeSet(TestVolume):
             self._volume.id,
             metadata
         )
+        self.volumes_mock.delete_metadata.assert_called_with(
+            self._volume.id,
+            self._volume.metadata.keys()
+        )
         self.volumes_mock.update_readonly_flag.assert_not_called()
         self.assertIsNone(result)
 
@@ -1036,8 +1231,8 @@ class TestVolumeShow(TestVolume):
         'bootable',
         'created_at',
         'display_description',
-        'display_name',
         'id',
+        'name',
         'properties',
         'size',
         'snapshot_id',
@@ -1054,8 +1249,8 @@ class TestVolumeShow(TestVolume):
             self._volume.bootable,
             self._volume.created_at,
             self._volume.display_description,
-            self._volume.display_name,
             self._volume.id,
+            self._volume.display_name,
             utils.format_dict(self._volume.metadata),
             self._volume.size,
             self._volume.snapshot_id,
@@ -1081,6 +1276,25 @@ class TestVolumeShow(TestVolume):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, data)
 
+    def test_volume_show_backward_compatibility(self):
+        arglist = [
+            '-c', 'display_name',
+            self._volume.id,
+        ]
+        verifylist = [
+            ('columns', ['display_name']),
+            ('volume', self._volume.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.get.assert_called_with(self._volume.id)
+
+        self.assertIn('display_name', columns)
+        self.assertNotIn('name', columns)
+        self.assertIn(self._volume.display_name, data)
+
 
 class TestVolumeUnset(TestVolume):
 
