@@ -208,6 +208,9 @@ class FakeVolumeClient(object):
         self.volume_types.resource_class = fakes.FakeResource(None, {})
         self.volume_type_access = mock.Mock()
         self.volume_type_access.resource_class = fakes.FakeResource(None, {})
+        self.volume_encryption_types = mock.Mock()
+        self.volume_encryption_types.resource_class = (
+            fakes.FakeResource(None, {}))
         self.restores = mock.Mock()
         self.restores.resource_class = fakes.FakeResource(None, {})
         self.qos_specs = mock.Mock()
@@ -224,6 +227,8 @@ class FakeVolumeClient(object):
         self.quota_classes.resource_class = fakes.FakeResource(None, {})
         self.consistencygroups = mock.Mock()
         self.consistencygroups.resource_class = fakes.FakeResource(None, {})
+        self.cgsnapshots = mock.Mock()
+        self.cgsnapshots.resource_class = fakes.FakeResource(None, {})
         self.auth_token = kwargs['token']
         self.management_url = kwargs['endpoint']
 
@@ -248,10 +253,7 @@ class TestVolume(utils.TestCommand):
 
 
 class FakeVolume(object):
-    """Fake one or more volumes.
-
-    TODO(xiexs): Currently, only volume API v2 is supported by this class.
-    """
+    """Fake one or more volumes."""
 
     @staticmethod
     def create_one_volume(attrs=None):
@@ -481,7 +483,7 @@ class FakeBackup(object):
         If backups list is provided, then initialize the Mock object with the
         list. Otherwise create one.
 
-        :param List volumes:
+        :param List backups:
             A list of FakeResource objects faking backups
         :param Integer count:
             The number of backups to be faked
@@ -547,6 +549,106 @@ class FakeConsistencyGroup(object):
 
         return consistency_groups
 
+    @staticmethod
+    def get_consistency_groups(consistency_groups=None, count=2):
+        """Note:
+
+        Get an iterable MagicMock object with a list of faked
+        consistency_groups.
+
+        If consistency_groups list is provided, then initialize
+        the Mock object with the list. Otherwise create one.
+
+        :param List consistency_groups:
+            A list of FakeResource objects faking consistency_groups
+        :param Integer count:
+            The number of consistency_groups to be faked
+        :return
+            An iterable Mock object with side_effect set to a list of faked
+            consistency_groups
+        """
+        if consistency_groups is None:
+            consistency_groups = (FakeConsistencyGroup.
+                                  create_consistency_groups(count))
+
+        return mock.Mock(side_effect=consistency_groups)
+
+
+class FakeConsistencyGroupSnapshot(object):
+    """Fake one or more consistency group snapshot."""
+
+    @staticmethod
+    def create_one_consistency_group_snapshot(attrs=None):
+        """Create a fake consistency group snapshot.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object with id, name, description, etc.
+        """
+        attrs = attrs or {}
+
+        # Set default attributes.
+        consistency_group_snapshot_info = {
+            "id": 'id-' + uuid.uuid4().hex,
+            "name": 'backup-name-' + uuid.uuid4().hex,
+            "description": 'description-' + uuid.uuid4().hex,
+            "status": "error",
+            "consistencygroup_id": 'consistency-group-id' + uuid.uuid4().hex,
+            "created_at": 'time-' + uuid.uuid4().hex,
+        }
+
+        # Overwrite default attributes.
+        consistency_group_snapshot_info.update(attrs)
+
+        consistency_group_snapshot = fakes.FakeResource(
+            info=copy.deepcopy(consistency_group_snapshot_info),
+            loaded=True)
+        return consistency_group_snapshot
+
+    @staticmethod
+    def create_consistency_group_snapshots(attrs=None, count=2):
+        """Create multiple fake consistency group snapshots.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :param int count:
+            The number of consistency group snapshots to fake
+        :return:
+            A list of FakeResource objects faking the
+            consistency group snapshots
+        """
+        consistency_group_snapshots = []
+        for i in range(0, count):
+            consistency_group_snapshot = (
+                FakeConsistencyGroupSnapshot.
+                create_one_consistency_group_snapshot(attrs)
+            )
+            consistency_group_snapshots.append(consistency_group_snapshot)
+
+        return consistency_group_snapshots
+
+    @staticmethod
+    def get_consistency_group_snapshots(snapshots=None, count=2):
+        """Get an iterable MagicMock object with a list of faked cgsnapshots.
+
+        If consistenct group snapshots list is provided, then initialize
+        the Mock object with the list. Otherwise create one.
+
+        :param List snapshots:
+            A list of FakeResource objects faking consistency group snapshots
+        :param Integer count:
+            The number of consistency group snapshots to be faked
+        :return
+            An iterable Mock object with side_effect set to a list of faked
+            consistency groups
+        """
+        if snapshots is None:
+            snapshots = (FakeConsistencyGroupSnapshot.
+                         create_consistency_group_snapshots(count))
+
+        return mock.Mock(side_effect=snapshots)
+
 
 class FakeExtension(object):
     """Fake one or more extension."""
@@ -665,7 +767,7 @@ class FakeQos(object):
         If qoses list is provided, then initialize the Mock object with the
         list. Otherwise create one.
 
-        :param List volumes:
+        :param List qoses:
             A list of FakeResource objects faking qoses
         :param Integer count:
             The number of qoses to be faked
@@ -738,7 +840,7 @@ class FakeSnapshot(object):
         If snapshots list is provided, then initialize the Mock object with the
         list. Otherwise create one.
 
-        :param List volumes:
+        :param List snapshots:
             A list of FakeResource objects faking snapshots
         :param Integer count:
             The number of snapshots to be faked
@@ -804,3 +906,101 @@ class FakeType(object):
             volume_types.append(volume_type)
 
         return volume_types
+
+    @staticmethod
+    def get_types(types=None, count=2):
+        """Get an iterable MagicMock object with a list of faked types.
+
+        If types list is provided, then initialize the Mock object with the
+        list. Otherwise create one.
+
+        :param List types:
+            A list of FakeResource objects faking types
+        :param Integer count:
+            The number of types to be faked
+        :return
+            An iterable Mock object with side_effect set to a list of faked
+            types
+        """
+        if types is None:
+            types = FakeType.create_types(count)
+
+        return mock.Mock(side_effect=types)
+
+    @staticmethod
+    def create_one_encryption_type(attrs=None):
+        """Create a fake encryption type.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object with volume_type_id etc.
+        """
+        attrs = attrs or {}
+
+        # Set default attributes.
+        encryption_info = {
+            "volume_type_id": 'type-id-' + uuid.uuid4().hex,
+            'provider': 'LuksEncryptor',
+            'cipher': None,
+            'key_size': None,
+            'control_location': 'front-end',
+        }
+
+        # Overwrite default attributes.
+        encryption_info.update(attrs)
+
+        encryption_type = fakes.FakeResource(
+            info=copy.deepcopy(encryption_info),
+            loaded=True)
+        return encryption_type
+
+
+class FakeQuota(object):
+    """Fake quota"""
+
+    @staticmethod
+    def create_one_vol_quota(attrs=None):
+        """Create one quota"""
+        attrs = attrs or {}
+
+        quota_attrs = {
+            'id': 'project-id-' + uuid.uuid4().hex,
+            'backups': 100,
+            'backup_gigabytes': 100,
+            'gigabytes': 10,
+            'per_volume_gigabytes': 10,
+            'snapshots': 0,
+            'volumes': 10}
+
+        quota_attrs.update(attrs)
+
+        quota = fakes.FakeResource(
+            info=copy.deepcopy(quota_attrs),
+            loaded=True)
+        quota.project_id = quota_attrs['id']
+
+        return quota
+
+    @staticmethod
+    def create_one_default_vol_quota(attrs=None):
+        """Create one quota"""
+        attrs = attrs or {}
+
+        quota_attrs = {
+            'id': 'project-id-' + uuid.uuid4().hex,
+            'backups': 100,
+            'backup_gigabytes': 100,
+            'gigabytes': 100,
+            'per_volume_gigabytes': 100,
+            'snapshots': 100,
+            'volumes': 100}
+
+        quota_attrs.update(attrs)
+
+        quota = fakes.FakeResource(
+            info=copy.deepcopy(quota_attrs),
+            loaded=True)
+        quota.project_id = quota_attrs['id']
+
+        return quota
